package sun.security.krb5;

import sun.security.krb5.internal.Krb5;
import sun.security.krb5.internal.TicketFlags;
import sun.security.util.DerValue;
import sun.security.krb5.internal.KrbCredInfo;
import java.io.IOException;
import sun.security.krb5.internal.HostAddresses;
import sun.security.krb5.internal.KDCOptions;
import sun.security.krb5.internal.KerberosTime;
import sun.security.krb5.internal.EncKrbCredPart;
import sun.security.krb5.internal.Ticket;
import sun.security.krb5.internal.KRBCred;

public class KrbCred
{
    private static boolean DEBUG;
    private byte[] obuf;
    private KRBCred credMessg;
    private Ticket ticket;
    private EncKrbCredPart encPart;
    private Credentials creds;
    private KerberosTime timeStamp;
    
    public KrbCred(Credentials credentials, Credentials credentials2, EncryptionKey encryptionKey) throws KrbException, IOException {
        this.obuf = null;
        this.credMessg = null;
        this.ticket = null;
        this.encPart = null;
        this.creds = null;
        this.timeStamp = null;
        PrincipalName client = credentials.getClient();
        PrincipalName server = credentials.getServer();
        PrincipalName server2 = credentials2.getServer();
        if (!credentials2.getClient().equals(client)) {
            throw new KrbException(60, "Client principal does not match");
        }
        KDCOptions kdcOptions = new KDCOptions();
        kdcOptions.set(2, true);
        kdcOptions.set(1, true);
        HostAddresses hostAddresses = null;
        if (server2.getNameType() == 3) {
            hostAddresses = new HostAddresses(server2);
        }
        this.credMessg = this.createMessage(new KrbTgsReq(kdcOptions, credentials, server, null, null, null, null, null, hostAddresses, null, null, null).sendAndGetCreds(), encryptionKey);
        this.obuf = this.credMessg.asn1Encode();
        return;
    }
    
    KRBCred createMessage(Credentials credentials, EncryptionKey encryptionKey) throws KrbException, IOException {
        EncryptionKey sessionKey = credentials.getSessionKey();
        PrincipalName client = credentials.getClient();
        client.getRealm();
        KrbCredInfo krbCredInfo = new KrbCredInfo(sessionKey, client, credentials.flags, credentials.authTime, credentials.startTime, credentials.endTime, credentials.renewTill, credentials.getServer(), credentials.cAddr);
        this.timeStamp = KerberosTime.now();
        return this.credMessg = new KRBCred(new Ticket[] { credentials.ticket }, new EncryptedData(encryptionKey, new EncKrbCredPart(new KrbCredInfo[] { krbCredInfo }, this.timeStamp, null, null, null, null).asn1Encode(), 14));
    }
    
    public KrbCred(byte[] data, EncryptionKey null_KEY) throws KrbException, IOException {
        this.obuf = null;
        this.credMessg = null;
        this.ticket = null;
        this.encPart = null;
        this.creds = null;
        this.timeStamp = null;
        this.credMessg = new KRBCred(data);
        this.ticket = this.credMessg.tickets[0];
        if (this.credMessg.encPart.getEType() == 0) {
            null_KEY = EncryptionKey.NULL_KEY;
        }
        EncKrbCredPart o = new EncKrbCredPart(new DerValue(this.credMessg.encPart.reset(this.credMessg.encPart.decrypt(null_KEY, 14))));
        this.timeStamp = o.timeStamp;
        KrbCredInfo krbCredInfo = o.ticketInfo[0];
        EncryptionKey key = krbCredInfo.key;
        PrincipalName pname = krbCredInfo.pname;
        TicketFlags flags = krbCredInfo.flags;
        KerberosTime authtime = krbCredInfo.authtime;
        KerberosTime starttime = krbCredInfo.starttime;
        KerberosTime endtime = krbCredInfo.endtime;
        KerberosTime renewTill = krbCredInfo.renewTill;
        PrincipalName sname = krbCredInfo.sname;
        HostAddresses caddr = krbCredInfo.caddr;
        if (KrbCred.DEBUG) {
            System.out.println(">>>Delegated Creds have pname=" + pname + " sname=" + sname + " authtime=" + authtime + " starttime=" + starttime + " endtime=" + endtime + "renewTill=" + renewTill);
        }
        this.creds = new Credentials(this.ticket, pname, null, sname, null, key, flags, authtime, starttime, endtime, renewTill, caddr);
    }
    
    public Credentials[] getDelegatedCreds() {
        return new Credentials[] { this.creds };
    }
    
    public byte[] getMessage() {
        return this.obuf;
    }
    
    static {
        KrbCred.DEBUG = Krb5.DEBUG;
    }
}
